<?php
/*
Plugin Name: Online Status inSL
Plugin URI: http://gwynethllewelyn.net/online-status-insl/
Version: 1.2.1
License: Simplified BSD License
Author: Gwyneth Llewelyn
Author URI: http://gwynethllewelyn.net/
Description: Shows your online status in the Second Life® world on a widget 

Copyright 2011 Gwyneth Llewelyn. All rights reserved.

Some tweaks by SignpostMarv

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY GWYNETH LLEWELYN ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GWYNETH LLEWELYN OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Gwyneth Llewelyn.

---

Based on code developed by Dave Doolin, http://website-in-a-weekend.net/extending-wordpress/wordpress-widget-plugin-super-easy-customization-tuesday-means-technical/

*/

if (!class_exists("online_status_insl"))
{
	if (!class_exists('WP_Http'))
		include_once(ABSPATH . WPINC . '/class-http.php');
 
	class online_status_insl extends WP_Widget
	{
		function online_status_insl()
		{
			$widget_ops = array('classname' => 'online_status_insl', 'description' => 'Online Status inSL' );
			$this->WP_Widget('online_status_insl_widget', 'Online Status inSL Widget', $widget_ops);
		}
 
		/* This is the code that gets displayed on the UI side,
		 * what readers see.
		 */
		function widget($args, $instance)
		{
			extract($args, EXTR_SKIP);
			echo $before_widget;
			$title = empty($instance['title']) ? '&nbsp;' : apply_filters('widget_title', $instance['title']);
 
			if (!empty($title)) { 
				echo $before_title,$title,$after_title;
			}
			
			// Code to check the in-world thingy and spew out results
			
			// get options
			
			$settings = maybe_unserialize(get_option('online_status_insl_settings'));
			
			$avatarLegacyName = $instance['avatar_name'];
			$PermURL = $settings[$avatarLegacyName]["PermURL"];
			
			print "<div class='osinsl'>";
			
			// see if the user wants us to place a profile picture for the avatar
			if ($instance['profile_picture'] != 'none')
			{
				$avatarNameSanitised = rawurlencode(strtolower(strtr($avatarLegacyName, " ", ".")));
?>			<a href='http://my.secondlife.com/<?php echo $avatarNameSanitised; ?>' target='_blank'><img class="osinsl-profile-picture" src="https://my-secondlife.s3.amazonaws.com/users/<?php echo $avatarNameSanitised; ?>/thumb_sl_image.png" width="80" height="80" alt="<?php echo $avatarLegacyName; ?>" align="<?php echo $instance['profile_picture']; ?>" valign="top"></a><br />
<?php
			}
			
			if ($PermURL) 
			{
				// check if we aren't hitting SL with too many requests, just do one
				//  every 60 seconds
				$timeRightNow = time();
				$dataResult = "";
				if ($timeRightNow - $settings[$avatarLegacyName]["timeStamp"] >= 60)
				{
					$request = new WP_Http;
					$result = $request->request($PermURL, 
						array(
							'method' => 'POST', 
							'body' => "command=avatar_data",
							'timeout' => 60
						));

					if (is_wp_error($result))
					{
						$dataResult = __("could not contact SL");
					}
					else
						$dataResult = $result['body'];
					
				}
				else // use cached value instead
				{
					$dataResult = $settings[$avatarLegacyName]["Status"];
				}		
					
				print "<span class='osinsl-before-status'>" . $instance['before_status'] . "</span>";
				if ($dataResult)
				{
	    			print "<span class='osinsl-status'>" . $dataResult . "</span><span class='osinsl-after-status'>" . $instance['after_status'] . "</span>";
	    			
	    			// change things back to options database
	    			$settings[$avatarLegacyName]["Status"] = $dataResult;    			
				}
				else
				{
	    			print "<span class='osinsl-problems'>" . $instance['having_problems'] . "</span>";
				}
			}
			else
			{
				print "<span class='osinsl-unconfigured'>" . $instance['unconfigured'] . "</span>";
			}
			print "</div>";

			// Avoid killing the database saving too often. Just update the settings
			//  if more than 60 seconds have elapsed 
			
			if ($timeRightNow - $settings[$avatarLegacyName]["timeStamp"] >= 60)
			{
   				$settings[$avatarLegacyName]["timeStamp"] = $timeRightNow;
   				update_option('online_status_insl_settings', $settings);
			}
			
			// return to widget handling code
            echo $after_widget;
		}
 
		function update($new_instance, $old_instance) {
			$instance = $old_instance;
			$instance['title'] = strip_tags($new_instance['title']);
			$instance['avatar_name'] = strip_tags($new_instance['avatar_name']);
			$instance['before_status'] = strip_tags($new_instance['before_status']);
			$instance['after_status'] = strip_tags($new_instance['after_status']);
			$instance['having_problems'] = strip_tags($new_instance['having_problems']);
			$instance['unconfigured'] = strip_tags($new_instance['unconfigured']);
			$instance['profile_picture'] = strip_tags($new_instance['profile_picture']);
			return $instance;
		}
 
		/* Back end, the interface shown in Appearance -> Widgets
		 * administration interface.
		 */
		function form($instance) {
			$defaults = array(
				'title' => __('Second Life Online Status'),
				'avatar_name' => __(''), 
				'before_status' => __('I am '), 
				'after_status' => __(' in Second Life.'),
				'having_problems' => __('having problems contacting RPC server...'),
				'unconfigured' => __('Please set up your in-world object first'),
				'profile_picture' => __('none')
			);
		
			$instance = wp_parse_args((array) $instance, $defaults);
					
			$title = strip_tags($instance['title']);
			
			// get the saved options; this will allow us to choose avatar names with
			//  registered in-world objects
			//  and assign this widget to one avatar name
			
			$settings = maybe_unserialize(get_option('online_status_insl_settings'));
			?>
 
<p>
<label for="<?php esc_attr_e($this->get_field_id('title')); ?>"><?php _e("Title"); ?>: 
    <input
	   class="widefat" id="<?php esc_attr_e($this->get_field_id('title')); ?>"
	   name="<?php esc_attr_e($this->get_field_name('title')); ?>" type="text"
	   value="<?php esc_attr_e($title); ?>" 
	/>
</label>
<label for="<?php esc_attr_e($this->get_field_id('avatar_name')); ?>"><?php _e("Avatar Name"); ?>:</label>
<select id="<?php esc_attr_e($this->get_field_id('avatar_name')); ?>" name="<?php esc_attr_e($this->get_field_name('avatar_name')); ?>" class="widefat" style="width:100%;">
<?php
	// now loop through all avatar names
	
	if ($settings && count($settings))
	{
		// POSSIBLE BUG: this might not work for legacy names != display names!!!!
		foreach ($settings as $oneWidget)
		{
	?><option <?php if ( $oneWidget["avatarDisplayName"] == $instance['avatar_name'] ) echo 'selected="selected"'; ?>><?php esc_attr_e($oneWidget["avatarDisplayName"]); ?></option>
	<?php
		}
	}
	else // never configured before
	{
	?>
	<option 'selected="selected"'>--<?php _e("Unconfigured"); ?>--</option>
	<?php
	}
	?>
</select>
<label for="<?php esc_attr_e($this->get_field_id('before_status')); ?>"><?php _e("Before status message"); ?>: 
    <input
	   class="widefat" id="<?php esc_attr_e($this->get_field_id('before_status')); ?>"
	   name="<?php esc_attr_e($this->get_field_name('before_status')); ?>" type="text"
	   value="<?php esc_attr_e($instance['before_status']); ?>" 
	/>
</label>
<label for="<?php esc_attr_e($this->get_field_id('after_status')); ?>"><?php _e("After status message"); ?>: 
    <input
	   class="widefat" id="<?php esc_attr_e($this->get_field_id('after_status')); ?>"
	   name="<?php esc_attr_e($this->get_field_name('after_status')); ?>" type="text"
	   value="<?php esc_attr_e($instance['after_status']); ?>" 
	/>
</label>
<label for="<?php esc_attr_e($this->get_field_id('having_problems')); ?>"><?php _e("Error message when communicating with SL"); ?>: 
    <input
	   class="widefat" id="<?php esc_attr_e($this->get_field_id('having_problems')); ?>"
	   name="<?php esc_attr_e($this->get_field_name('having_problems')); ?>" type="text"
	   value="<?php esc_attr_e($instance['having_problems']); ?>" 
	/>
</label>
<label for="<?php esc_attr_e($this->get_field_id('unconfigured')); ?>"><?php _e("Widget not configured message"); ?>: 
    <input
	   class="widefat" id="<?php esc_attr_e($this->get_field_id('unconfigured')); ?>"
	   name="<?php esc_attr_e($this->get_field_name('unconfigured')); ?>" type="text"
	   value="<?php esc_attr_e($instance['unconfigured']); ?>" 
	/>
</label>
<label for="<?php esc_attr_e($this->get_field_id('profile_picture')); ?>"><?php _e("Profile picture?"); ?></label>
<select id="<?php esc_attr_e($this->get_field_id('profile_picture')); ?>" name="<?php esc_attr_e($this->get_field_name('profile_picture')); ?>" class="widefat">
	<option <?php if ( 'none' == $instance['profile_picture'] ) echo 'selected="selected"'; ?>>none</option>
	<option <?php if ( 'center' == $instance['profile_picture'] ) echo 'selected="selected"'; ?>>center</option>
	<option <?php if ( 'left' == $instance['profile_picture'] ) echo 'selected="selected"'; ?>>left</option>
	<option <?php if ( 'right' == $instance['profile_picture'] ) echo 'selected="selected"'; ?>>right</option>
</select>
</p>
			<?php echo "\n<!-- Widget instance is "; var_dump($instance); echo "-->\n";
		}			
	}
 
	function online_status_insl_widget_init()
	{
		register_widget('online_status_insl');
	}

	function online_status_insl_widget_activate()
	{
		// file_put_contents(ABSPATH . 'wp-content/uploads/channel.inc', "");
	}

	function online_status_insl_widget_deactivate()
	{
		// clean up options on database
		delete_option('online_status_insl_settings');
		// sanitize
		unregister_setting( 'online_status_insl', 'online_status_insl_settings' );
		unlink(ABSPATH . 'wp-content/uploads/channel.inc'); // keep it tidy
	}
	
	function online_status_insl_admin_menu_options()
	{
		add_options_page('Online Status inSL', 'Online Status inSL', 1,
			'online_status_insl', 'online_status_insl_menu');
	}
	
	function online_status_insl_menu()
	{
?>
<div class="wrap">
<h2>Online Status inSL</h2>
<?php	
	$settings = maybe_unserialize(get_option('online_status_insl_settings'));

		// Check if we have to delete some of the online indicators
		if ($_POST["online_status_insl_form"])
		{
			check_admin_referer('delete-online-status-indicators');
					
			// emit "updated" class showing we have deleted some things
?>
	<div id="message-updated" class="updated"><p><?php _e("Online status indicators <strong>deleted</strong>"); ?><br /><br />
<?php	
			// Aye, we should have some things to delete first
		
			// loop through settings to delete; we get objectKeys and not the avatar name
			// _e("Dumping "); echo "\$_POST: "; var_dump($_POST);
			
			$newSettings = array();
			foreach ($settings as $onlineStatusIndicator)
			{
				if (in_array($onlineStatusIndicator["objectKey"], $_POST["deletedStatusIndicators"]))
				{
					_e("Deleting "); 
					echo $onlineStatusIndicator["avatarDisplayName"], ", ",
						__("Object Name: "), $onlineStatusIndicator["objectName"], 
						" (", $onlineStatusIndicator["objectKey"], "), ",
						__("Location: "), $onlineStatusIndicator["objectRegion"],
						"<br />";
				}
				else 
				{
					$newSettings[$onlineStatusIndicator["avatarDisplayName"]] = $onlineStatusIndicator;
					// _e("Keeping "); echo $onlineStatusIndicator["avatarDisplayName"], ",",
					//	$onlineStatusIndicator["objectKey"], "<br>";
				}
			}
			$settings = $newSettings;
			
			// update options with new settings; gets serialized automatically
			update_option('online_status_insl_settings', $settings);
?>
			</p></div>
<?php
		}

		_e("Please create an object in Second Life on a plot owned by you, and drop the following script inside:"); ?>
<p>
<hr />
<textarea name="osinsl-lsl-script" cols="120" rows="12" readonly style="font-family: monospace">
// Code by Gwyneth Llewelyn to show online status and let it
// be retrieved by external calls
// Global Variables
key avatar;
string avatarName;
// Things we will receive from the dataserver
string onlineStatus = "status unknown"; // when the dataserver is slow, this will remain unset
string displayName = "(???)";
string dateBorn = "1970-01-01"; // avatar rezday; set to epoch if not retrieved
key onlineStatusRequest;    // to request items from the dataserver
key dateBornStatusRequest;
key displayNameStatusRequest;
key webResponse;    // to send the PermURL to the webserver
string objectVersion = "1.2.1";

// modified by SignpostMarv
string http_host = "<?php esc_attr_e($_SERVER['HTTP_HOST']); ?>";


default
{
    state_entry()
    {
        llSetText("Waiting for data server...", <0.8, 0.1, 0.1>, 1.0);
        llSetTimerEvent(60.0);  // call a timer event every 60 seconds.
        avatar = llGetOwner();
        avatarName = llKey2Name(avatar);
        llOwnerSay("Registering with your blog and retrieving user data from the SL dataserver...");
        onlineStatusRequest = llRequestAgentData(avatar, DATA_ONLINE);
        dateBornStatusRequest = llRequestAgentData(avatar, DATA_BORN);
        displayNameStatusRequest = llRequestAgentData(avatar, DATA_NAME);
        // wait a bit, dataserver is sometimes slow
        llSleep(10.0);
        llSetText("Requesting PermURL from SL...", <0.8, 0.8, 0.1>, 1.0);
        llRequestURL();  // this sets the object up to accept external HTTP-in calls
    }   

    on_rez(integer startParam)
    {
        llResetScript();
    }
    
    touch(integer howmany)  // Allow owner to reset this
    {
        if (llDetectedKey(0) == avatar)
        {
            llResetScript();
        }
    }

    timer()
    {
        // Every 60 seconds, retrieve the online status from the SL database server
        //  The other things will remain static (e.g. original name and date born)
        onlineStatusRequest = llRequestAgentData(avatar, DATA_ONLINE);
        llSetText(avatarName + " is " + onlineStatus, ZERO_VECTOR, 1.0);    // set the hover text
    }

    changed(integer what)
    {
        if (what & CHANGED_OWNER)
            llResetScript();    // make sure the new owner gets a fresh PermURL!
        if (what & (CHANGED_REGION | CHANGED_REGION_START | CHANGED_TELEPORT) )
        {
            llSetText("Requesting PermURL from SL...", <0.8, 0.8, 0.1>, 1.0);
            llRequestURL();
        }
    }

    dataserver(key queryid, string data)
    {
        // this is what SL returns to us when we call llRequestAgentData()
        if (queryid == onlineStatusRequest) // check if it’s the correct request
        {
            if (data == "1")    // online status is just 0 or 1
                onlineStatus = "online";
            else
                onlineStatus = "offline";
        }
        else if (queryid == dateBornStatusRequest)
        {
            dateBorn = data;
        }
        else if (queryid == displayNameStatusRequest)
        {
            displayName = data;
        }
    }

    // This is just to catch that our website has the widget active
    http_response(key request_id, integer status, list metadata, string body)
    {
        if (request_id == webResponse)
        {
            if (status == 200)
            {
                llOwnerSay("PermURL sent to gateway! Msg. id is " + body);
            }
            else if (status == 499)
            {
                llOwnerSay("Timeout waiting for gateway! Your PermURL might still be sent, please be patient");
            }
            else
            {
                llOwnerSay("PermURL NOT sent. Status was " + (string)status + "; error message: " + body);
            }
            llSetText("", <0.0, 0.0, 0.0>, 1.0);
        }
    }

    // These are requests made from our blog to this object
    http_request(key id, string method, string body)
    {
        if (method == URL_REQUEST_GRANTED)
        {
            llSetText("Sending PermURL to blog...", <0.6, 0.6, 0.1>, 1.0);

            string avatarName = llKey2Name(llGetOwner());
            string message = "avatar_name=" + llEscapeURL(displayName) + 
                "&object_version=" + llEscapeURL(objectVersion) +
                "&dateBorn=" + dateBorn +
                "&PermURL=" + llEscapeURL(body);
            // llOwnerSay("DEBUG: Message to send to blog is: " + message);
            webResponse = llHTTPRequest("http://" + http_host + "/wp-content/plugins/online-status-insl/save-channel.php",
                [HTTP_METHOD, "POST", HTTP_MIMETYPE, "application/x-www-form-urlencoded"], 
                message);            
            
        }
        else if (method == "POST" || method == "GET")
        {
            if (body == "") // weird, no request
            {
                llHTTPResponse(id, 403, "Empty message received");
            }
            else // tries to get agent data
            {
                // send stored message back
                llHTTPResponse(id, 200, onlineStatus);
            }
        } 
    }    
}
</textarea>
<p>
<hr />
<?php _e("Then you can drag the appropriate widget on your sidebar."); ?>

<?php 
if (count($settings) > 0)
{
	?><h2><?php _e("Current avatars being tracked"); ?>:</h2>
<?php


	// $url = $_SERVER['PHP_SELF'] . "?page=" . $_GET["page"]. "&action=delete-online-status-indicators";
	// $action = "online-status-insl-delete-online-status-indicators";
	// $link = wp_nonce_url($url, $action);
?>

<form method='post' id="online_status_insl_form">
<?php wp_nonce_field("delete-online-status-indicators"); ?>
<!--<input type="hidden" name="action" value="delete-online-status-indicators">-->
<table class="wp-list-table widefat fixed" cellspacing="0">
	<thead>
		<tr>
			<th scope='col' class='manage-column column-name'><?php _e("Avatar Display Name"); ?></th>
			<th scope='col' class='manage-column column-status'><?php _e("Status"); ?></th>
			<th scope='col' class='manage-column column-permurl'><?php _e("PermURL"); ?></th>
			<th scope='col' class='manage-column column-avatarkey'><?php _e("Avatar Key"); ?></th>
			<th scope='col' class='manage-column column-objname'><?php _e("Object Name"); ?></th>
			<th scope='col' class='manage-column column-objkey'><?php _e("Object Key"); ?></th>
			<th scope='col' class='manage-column column-obversion'><?php _e("Object Version"); ?></th>
			<th scope='col' class='manage-column column-location'><?php _e("Location"); ?></th>
			<th scope='col' class='manage-column column-date'><?php _e("Last time checked"); ?></th>
			<th scope='col' class='manage-column column-cb check-column'></th>
		</tr>
	</thead>
	<tbody>
<?php
		foreach ($settings as $oneWidget)
		{
			// needed for linking to SL's structure
			$avatarNameSanitised = rawurlencode(strtolower(strtr($oneWidget["avatarDisplayName"], " ", ".")));
?>
<tr class="format-default <?php echo ($alternate ? "" : "alternate"); $alternate = !$alternate; ?>">
			<td><a href='http://my.secondlife.com/<?php echo $avatarNameSanitised; ?>' target='_blank'><?php echo $oneWidget["avatarDisplayName"]; ?>
			<img src="https://my-secondlife.s3.amazonaws.com/users/<?php echo $avatarNameSanitised; ?>/thumb_sl_image.png" width="80" height="80" alt="<?php echo $oneWidget["avatarDisplayName"]; ?>" align="center" valign="bottom"></a></td>
			<td><?php echo $oneWidget["Status"]; ?></td>
			<td><?php echo $oneWidget["PermURL"]; ?></td>
			<td><?php echo $oneWidget["avatarKey"]; ?></td>
			<td><?php echo $oneWidget["objectName"]; ?></td>
			<td><?php echo $oneWidget["objectKey"]; ?></td>
			<td><?php echo $oneWidget["objectVersion"]; ?></td>
			<td>
<?php 
		// parse name of the region and coordinates to create a link to maps.secondlife.com
		$regionName = substr($oneWidget["objectRegion"], 0, strpos($oneWidget["objectRegion"], "(") - 1);
		$coords = trim($oneWidget["objectLocalPosition"], "() \t\n\r");
		$xyz = explode(",", $coords);
		
		printf('<a href="http://maps.secondlife.com/secondlife/%s/%F/%F/%F?title=%s&amp;msg=%s&amp;img=%s" target="_blank">%s (%d,%d,%d)</a>',
			$regionName, $xyz[0], $xyz[1], $xyz[2], 
			rawurlencode($oneWidget["objectName"]),
			rawurlencode(__("Online Status Indicator for ") . $oneWidget["avatarDisplayName"]),
			rawurlencode("http://s.wordpress.org/about/images/logos/wordpress-logo-stacked-rgb.png"),
			$regionName, $xyz[0], $xyz[1], $xyz[2]); ?></td>
			<td class="date column-date"><?php echo date("Y M j H:i:s", $oneWidget["timeStamp"]); ?></td>
			<td ><input type="checkbox" name="deletedStatusIndicators[]" value="<?php echo $oneWidget["objectKey"]; ?>" /></td>
		</tr>
<?		
		}
?>
	</tbody>
</table>
<input type='submit' class='button-primary alignleft' name='online_status_insl_form' value='<?php _e('Delete'); ?>' />
</form>
<?php
	} // if settings not empty
	else
	{
?>
<br /><br /><strong>
<?php
		_e("No avatars are being tracked.");
?></strong>
<?php
	}
	}
	
	// Add a settings group, which hopefully makes it easier to delete later on
	function online_status_insl_register_settings()
	{
		register_setting('online_status_insl', 'online_status_insl_settings');
		// it's a huge serialised array for now, stored as a WP option in the database; 
		//  if performance drops, this might change in the future
	}
	
	// Deals with shortcodes. For now, we just have [osinsl avatar="AvatarName"]
	function online_status_insl_shortcode($atts)
	{
		extract( shortcode_atts( array(
      		'avatar' => '(???)',	// assigns $avatar to name if it exists, and provides a default of (???) which is supposed *not* to exist
      		'picture' => 'none',	// emits picture tags, can be center/right/left/ etc.
      		'status'  => 'on',		// emits no status, just the picture (or nothing)
      		'profilelink' => 'off'	// puts links to web profile if picture active
      		), $atts ) );
      	// search for the avatar name
      	$settings = maybe_unserialize(get_option('online_status_insl_settings'));
      	
      	// store things in a return value; add class attributes to allow styling
      	$returnValue = "<span class='osinsl-shortcode' id='osinsl-shortcode-"
      		. strtolower(strtr($avatar, " ", "-"))
      		. "'>";
      	
      	$avatarNameSanitised = rawurlencode(strtolower(strtr($avatar, " ", ".")));
      	
      	if ($settings) // did we find anything at all??
      	{
      		if ($settings[$avatar]) // just see if we have an entry for this
      		{
      			if ($picture != "none")
      			{
      				if ($profilelink != 'off')
      					$returnValue .= "<a href='http://my.secondlife.com/" . $avatarNameSanitised . "' target='_blank'>";
      				$returnValue .= '<img class="osinsl-profile-picture" src="https://my-secondlife.s3.amazonaws.com/users/' 
      					. $avatarNameSanitised . '/thumb_sl_image.png" width="80" height="80" alt="'
      					. $avatar . '" align="' 
      					. $picture. '" valign="bottom">';
      				if ($profilelink != 'off')
      					$returnValue .= "</a>";
      			}
      			if ($status != "off") $returnValue .= $settings[$avatar]['Status'];
      		}
      		else $returnValue .= __("No widget configured for ") . $avatar;
      	}
      	else $returnValue .=  __("No avatars being tracked");
      				
      	$returnValue .= "</span>";
      	
      	return $returnValue;
	}

	add_action('widgets_init', 'online_status_insl_widget_init');
	add_action('admin_menu', 'online_status_insl_admin_menu_options');
 	register_activation_hook(__FILE__, 'online_status_insl_widget_activate');
 	register_deactivation_hook(__FILE__, 'online_status_insl_widget_deactivate');
 	add_action('admin_init', 'online_status_insl_register_settings' );
 	add_shortcode('osinsl', 'online_status_insl_shortcode');
}
 
$wpdpd = new online_status_insl();
 
?>